from sasktran2 import pysasktran2 as skinternal
import abc


class ConfigBase(abc.ABC):
    @abc.abstractmethod
    def internal_config(self) -> skinternal.Config:
        pass


class Config(ConfigBase):
    def __init__(self):
        self._nstokes = 1
        self._nthreads = 1
        self._msmode = 0
        self._nstreams = 16
        self._do_spherical_iterations = 0

    @property
    def nstokes(self):
        return self._nstokes

    @nstokes.setter
    def nstokes(self, ns: int):
        if ns != 1 and ns != 3:
            raise ValueError('numstokes must be either 1 or 3')

        self._nstokes = ns

    @property
    def nstreams(self):
        return self._nstreams

    @nstreams.setter
    def nstreams(self, streams: int):
        self._nstreams = streams

    @property
    def ms_mode(self):
        return self._msmode

    @ms_mode.setter
    def ms_mode(self, mode: int):
        self._msmode = mode

    @property
    def do_spherical_iterations(self):
        return self._do_spherical_iterations

    @do_spherical_iterations.setter
    def do_spherical_iterations(self, iter: int):
        self._do_spherical_iterations = iter

    def internal_config(self) -> skinternal.Config:
        config = skinternal.Config(self._nstokes, self._nthreads)

        config.set_ms_mode(self._msmode)
        config.set_num_do_spherical_iterations(self._do_spherical_iterations)
        config.set_num_do_streams(self._nstreams)

        return config
