from sasktran2 import pysasktran2 as skinternal
from .geometry import GeometryBase
from .atmosphere import AtmosphereBase
from .config import ConfigBase
from .viewinggeometry import ViewingGeometryBase
from copy import copy


class Sasktran:
    def __init__(self, config: ConfigBase, geometry: GeometryBase, viewing_geometry: ViewingGeometryBase):
        self._config = config
        self._geometry = geometry
        self._viewing_geometry = viewing_geometry

        self._internal_config = config.internal_config()
        self._internal_geometry = geometry.internal_geometry()
        self._internal_viewing_geometry = viewing_geometry.internal_viewing_geometry()

        self._engine = skinternal.Sasktran(self._internal_geometry,
                                           self._internal_viewing_geometry,
                                           self._internal_config)

    def calculate(self, atmosphere: AtmosphereBase,
                  output: skinternal.Output = None,
                  output_fmt='raw'
                  ):
        if output is None:
            # Use the default output format
            output = skinternal.Output(self._internal_config.num_stokes())

        self._engine.calculate_radiance(atmosphere.internal_atmosphere(
            self._config,
            self._geometry,
            self._viewing_geometry
        ), output)

        if output_fmt.lower() == 'raw':
            return {'radiance': copy(output.dense_radiance_view()),
                    'wf': copy(output.dense_wf_view())}
        else:
            raise ValueError('invalid output_fmt value of {}'.format(output_fmt))
