import sasktran2.pysasktran2 as skinternal
import abc
import numpy as np


class ViewingGeometryBase(abc.ABC):
    @abc.abstractmethod
    def internal_viewing_geometry(self) -> skinternal.ViewingGeometry:
        pass


class ViewingGeometry(ViewingGeometryBase):
    def __init__(self, viewing_geometry: skinternal.ViewingGeometry):
        self._viewing_geometry = viewing_geometry

    @classmethod
    def from_tangent_altitudes(cls, tangent_altitudes: np.array,
                               relative_azimuth: np.array,
                               observer_altitude: np.array,
                               theta: np.array = None,
                               phi: np.array = None
                               ):

        if theta is None:
            theta = np.zeros_like(tangent_altitudes)
        if phi is None:
            phi = np.zeros_like(tangent_altitudes)

        viewing_geo = skinternal.ViewingGeometry()

        for i in range(len(tangent_altitudes)):
            viewing_geo.add_tangent_ray(float(tangent_altitudes[i]),
                                        float(relative_azimuth[i]),
                                        float(observer_altitude[i]),
                                        float(theta[i]),
                                        float(phi[i]))

        return ViewingGeometry(viewing_geo)

    def internal_viewing_geometry(self) -> skinternal.ViewingGeometry:
        return self._viewing_geometry
