import numpy as np
import sasktran2.pysasktran2 as sk
from sasktran2.tests.util import default_pure_scattering_atmosphere, default_viewing_geometry, extinction_ssa_central_difference, validate_wfs


def test_wf_singlescatter_scalar():
    cos_sza = 0.8
    alt_grid = np.arange(0, 100001, 1000)
    atmo_dict = default_pure_scattering_atmosphere(alt_grid)

    geo = sk.Geometry(cos_sza, np.pi/2, 6371000, alt_grid)
    config = sk.Config(1)
    config.set_ms_mode(0)

    viewing_geo = default_viewing_geometry()

    atmosphere = sk.Atmosphere(1, alt_grid.size, 1, 16)

    atmosphere.set_manual_extinction(atmo_dict['extinction'])
    atmosphere.set_manual_ssa(atmo_dict['ssa'])
    atmosphere.set_manual_legendre(atmo_dict['legendre'])

    # Calculate legendre derivatives to the second and third terms

    leg_deriv = np.zeros((atmo_dict['legendre'].shape[0], 2, atmo_dict['legendre'].shape[1], atmo_dict['legendre'].shape[2]))

    leg_deriv[0, 0, :, 4] = 1
    leg_deriv[0, 1, :, 2] = 1

    atmosphere.set_manual_legendre_derivatives(leg_deriv)

    engine = sk.Sasktran(geo, viewing_geo, config)

    wfs = extinction_ssa_central_difference(len(alt_grid), atmo_dict, atmosphere, engine, 1, leg_deriv=leg_deriv)

    validate_wfs(wfs['analytic'], wfs['numerical'], len(alt_grid), 2)